% Auxiliary function for unitary_QZ_ds.m;
% given quasiseparable generators, it computes 
% the corresponding matrix. Useful to check that 
% everything is going ok!
%
% Matrix B is supposed to be upper triangular.
% We have explicit diagonal entries
% and quasiseparable generators for the strictly
% upper triangular part.
%
% Please note that no check is performed on 
% input size. We just assume it's correct!

function B=compute_matrix_B(d,g,h,b,p,q)
n=length(p); % this is hopefully the size of A
B=zeros(n);
% we first compute the unitary part; later
% we will add the rank one correction

% fill diagonal
for i=1:n
    B(i,i)=d(i);
end

% fill strict triangular upper part of B
% (check for order of generators)
[g1,g2]=size(g);
if g1<g2
    t=g2;
    g2=g1;
    g1=t;
end
if g2>1
   for i=1:n-1
    for j=i+1:n
        x=h(:,j);
        for k=j-1:-1:i+1
            x=b(:,:,k)*x;
        end
        B(i,j)=g(i,:)*x-p(i)*q(j);
    end
   end
else
    for i=1:n-1
    for j=i+1:n
        x=h(j);
        for k=j-1:-1:i+1
            x=b(k)*x;
        end
        B(i,j)=g(i)*x-p(i)*q(j);
    end
    end
end
    


